from django.contrib import admin

from eggnog.models import *

class UpdateAdmin(admin.ModelAdmin):
    """
    Administration interface for the package updates.
    """
    list_display = ('cheeseshop', 'updateable', 'installed', 'available', 'checked')
    ordering = ['-checked']
    list_per_page = 100
    search_fields = ['package']
    readonly_fields = ['package', 'installed', 'available', 'checked']
    exclude = None
    max_num = 0

    def has_add_permission(self, request, obj=None):
        """
        Disable adding.
        """
        return False

    def has_delete_permission(self, request, obj=None):
        """
        Disable deleting.
        """
        return False

    def get_actions(self, request):
        """
        Removes all the bulk actions as there are none.
        """
        return []

admin.site.register(Update, UpdateAdmin)



from pkg_resources import *

from django.db import models

class Update(models.Model):
    """
    Model for storing the updates
    """
    package = models.CharField(max_length=2000, null=False, unique=True)
    installed = models.CharField(max_length=16, null=False)
    available = models.CharField(max_length=16, null=False)
    checked = models.DateTimeField(auto_now_add=True, null=False)

    def __unicode__(self):
        """
        String representation method.
        """
        return self.package + ' [' + self.installed + ']'

    class Meta:
        """
        Meta.
        """
        ordering = ['-checked']

    def updateable(self):
        """
        Custom property indicating whether an update is available.
        """
        return True if parse_version(self.installed) < parse_version(self.available) else False

    updateable.boolean = True

    def cheeseshop(self):
        """
        Returns a URL pointing to the package on PyPi
        """
        return '<a href="http://pypi.python.org/pypi/%s/">%s</a>' % (self.package, self.package)

    cheeseshop.allow_tags = True



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Update'
        db.create_table('eggnog_update', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('package', self.gf('django.db.models.fields.CharField')(unique=True, max_length=2000)),
            ('installed', self.gf('django.db.models.fields.CharField')(max_length=16)),
            ('available', self.gf('django.db.models.fields.CharField')(max_length=16)),
            ('checked', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('eggnog', ['Update'])


    def backwards(self, orm):
        # Deleting model 'Update'
        db.delete_table('eggnog_update')


    models = {
        'eggnog.update': {
            'Meta': {'ordering': "['-checked']", 'object_name': 'Update'},
            'available': ('django.db.models.fields.CharField', [], {'max_length': '16'}),
            'checked': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'installed': ('django.db.models.fields.CharField', [], {'max_length': '16'}),
            'package': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '2000'})
        }
    }

    complete_apps = ['eggnog']



import sys
from setuptools import setup, find_packages

setup(
    name = "django-eggnog",
    package_data = {
        'eggnog': [
            'README.rst',
            'LICENSE.txt',
        ],
    },
    author = "Mridang Agarwalla",
    author_email = "mridang.agarwalla@gmail.com",
    maintainer = "Mridang Agarwalla",
    maintainer_email = "mridang.agarwalla@gmail.com",
    download_url='http://github.com/mridang/django-eggnog/downloads',
    bugtrack_url='http://github.com/mridang/django-eggnog/issues',
    description = "Displays available egg updates from PyPi",
    long_description=open('README.rst').read(),
    url = "http://github.org/mridang/django-eggnog",
    classifiers = [
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    packages = [
        'eggnog',
        'eggnog.migrations',
        'eggnog.management.commands',
    ],
    zip_safe = True,
    license = "BSD License",
    install_requires = [
        'Django>=1.4',
        'south>=0.7.2',
        'yolk',
        'apscheduler',
    ],
    version = '0.0.1',
)



def periodicallycheckforupdates():
    """
    Initialises and starts the advanced Python scheduler
    """
    try:
        from apscheduler.scheduler import Scheduler
    except ImportError, i:
        return
    else:
        sched = Scheduler()

        @sched.interval_schedule(seconds=86400)
        def __check():
            """
            Periodically checks for updates
            """
            try:
                from django.core import management
                management.call_command('checkupdates', verbosity=0, interactive=False)
            except Exception, e:
                print e

        sched.start()



from threading import Thread
from pkg_resources import *

from django.core.management.base import BaseCommand, CommandError

from yolk.setuptools_support import get_pkglist
from yolk.yolklib import get_highest_version, Distributions
from yolk.pypi import CheeseShop

from eggnog.models import Update

class Command(BaseCommand):
    """
    Custom management command for checking for package updates.
    """
    help = 'Checks for package updates from PyPi'

    threads = []
    dists = Distributions()
    pypi = CheeseShop()

    def __init__(self, *args, **kwargs):
        """
        Initializer for the management commands to flush stale data.
        """
        super(Command, self).__init__(*args, **kwargs)

        Update.objects.all().delete()

    def handle(self, *args, **options):
        """
        Main management command method that starts the checking process.
        """
        print "Checking for updates from PyPi"

        for pkg in get_pkglist():

            for (dist, active) in self.dists.get_distributions("all", pkg, self.dists.get_highest_installed(pkg)):
                 thread = Thread(target=self.__check_pypi, args=(dist.project_name, dist.version))
                 self.threads.append(thread)
                 thread.start()

        for thread in self.threads:
            thread.join()

    def __check_pypi(self, name, current):
        """
        Queries PyPi for updates
        """
        (package, versions) = self.pypi.query_versions_pypi(name)

        if versions:
            newest = get_highest_version(versions)
            if newest != current:
                if parse_version(current) < parse_version(newest):
                    print " * Updates for %s are available. You have %s and the latest is %s." % (package, current, newest)
                else:
                    print " * No updates are available for %s." % (package) 
            Update.objects.create(package=package, installed=current, available=newest)
                
